# AWS Amplify Module for devops-ai

The `devops_ai_amplify` module provides a comprehensive CLI interface for deploying React applications (and other frontend frameworks) to AWS Amplify from GitHub repositories.

## Features

- 🚀 **Zero-config deployment** to AWS Amplify
- 🌿 **Branch-specific deployments** with automatic builds
- 🎯 **Preview deployments** for pull requests
- 🔧 **Framework-specific configurations** (React, Next.js, Vue, Angular)
- 📱 **Responsive design** support
- 🔒 **SSL certificates** and custom domain management
- 🔄 **CI/CD integration** with GitHub Actions
- 📊 **Build monitoring** and status tracking

## Installation

The module is included with the main `devops-ai` package:

```bash
pip install devops-ai
```

## Quick Start

### 1. Create an Amplify App

```bash
devops-ai amplify create-app \
  --name my-react-app \
  --repo https://github.com/username/my-react-app \
  --branch main \
  --region us-east-1
```

### 2. Configure for Your Framework

```bash
devops-ai amplify configure-app \
  --framework react \
  --force
```

### 3. Deploy a Branch

```bash
devops-ai amplify deploy-branch \
  --branch main \
  --wait
```

## Commands Reference

### App Management

#### `create-app`

Create a new Amplify app and connect it to a GitHub repository.

```bash
devops-ai amplify create-app \
  --name my-app \
  --repo https://github.com/username/repo \
  --branch main \
  --region us-east-1 \
  --token YOUR_GITHUB_TOKEN
```

**Options:**
- `--name, -n`: App name
- `--repo, -r`: GitHub repository URL
- `--branch, -b`: Default branch (default: main)
- `--region`: AWS region (default: us-east-1)
- `--token, -t`: GitHub access token (optional)
- `--force, -f`: Overwrite existing configuration

#### `list-apps`

List all Amplify apps in the specified region.

```bash
devops-ai amplify list-apps --region us-east-1
```

#### `delete-app`

Delete an Amplify app.

```bash
devops-ai amplify delete-app \
  --app-id d1234567890 \
  --region us-east-1 \
  --force
```

**Options:**
- `--app-id, -a`: Amplify app ID
- `--region`: AWS region
- `--force, -f`: Skip confirmation

#### `status`

Show status of an Amplify app.

```bash
devops-ai amplify status --app-id d1234567890
```

### Branch Management

#### `create-branch`

Create a new branch for deployment.

```bash
devops-ai amplify create-branch \
  --branch feature/new-feature \
  --app-id d1234567890 \
  --auto-build \
  --pr-preview
```

**Options:**
- `--branch, -b`: Branch name
- `--app-id, -a`: Amplify app ID
- `--region`: AWS region
- `--auto-build`: Enable auto build (default: true)
- `--pr-preview`: Enable pull request preview (default: true)

#### `list-branches`

List all branches for an Amplify app.

```bash
devops-ai amplify list-branches --app-id d1234567890
```

#### `delete-branch`

Delete a branch from an Amplify app.

```bash
devops-ai amplify delete-branch \
  --branch feature/old-feature \
  --app-id d1234567890 \
  --force
```

#### `deploy-branch`

Deploy a specific branch to Amplify.

```bash
devops-ai amplify deploy-branch \
  --branch main \
  --app-id d1234567890 \
  --wait
```

**Options:**
- `--branch, -b`: Branch to deploy
- `--app-id, -a`: Amplify app ID
- `--region`: AWS region
- `--wait, -w`: Wait for deployment to complete (default: true)

### Configuration

#### `configure-app`

Configure Amplify app with framework-specific settings.

```bash
devops-ai amplify configure-app \
  --framework react \
  --app-id d1234567890 \
  --force
```

**Options:**
- `--app-id, -a`: Amplify app ID
- `--region`: AWS region
- `--framework, -f`: Framework (react, nextjs, vue, angular)
- `--force, -f`: Overwrite existing files

## Framework Support

### React

Standard React applications created with Create React App.

**Build Configuration:**
- Install dependencies: `npm ci`
- Build command: `npm run build`
- Output directory: `build/`

### Next.js

Next.js applications with static export.

**Build Configuration:**
- Install dependencies: `npm ci`
- Build command: `npm run build && npm run export`
- Output directory: `out/`

### Vue.js

Vue.js applications.

**Build Configuration:**
- Install dependencies: `npm ci`
- Build command: `npm run build`
- Output directory: `dist/`

### Angular

Angular applications.

**Build Configuration:**
- Install dependencies: `npm ci`
- Build command: `npm run build --prod`
- Output directory: `dist/`

## Configuration Files

### amplify.yml

The build configuration file generated by the module:

```yaml
version: 1
frontend:
  phases:
    preBuild:
      commands:
        - echo "Installing dependencies..."
        - npm ci
    build:
      commands:
        - echo "Building React app..."
        - npm run build
  artifacts:
    baseDirectory: build
    files:
      - '**/*'
  cache:
    paths:
      - node_modules/**/*
  customHeaders:
    - pattern: '**/*'
      headers:
        - key: 'Cache-Control'
          value: 'max-age=0, no-cache, no-store, must-revalidate'
```

### _redirects

For Single Page Application routing:

```
/*    /index.html   200
```

### amplify-config.json

Local configuration file:

```json
{
  "app": {
    "name": "my-react-app",
    "id": "d1234567890",
    "repository": "https://github.com/username/my-react-app",
    "branch": "main"
  },
  "aws": {
    "region": "us-east-1"
  },
  "build": {
    "framework": "react",
    "buildCommand": "npm run build",
    "outputDirectory": "build",
    "installCommand": "npm ci"
  }
}
```

## Environment Variables

Configure environment variables in the Amplify console or via the AWS CLI:

```bash
# Example for React apps
REACT_APP_API_URL=https://api.example.com
REACT_APP_ENVIRONMENT=production
NODE_ENV=production

# Example for Next.js apps
NEXT_PUBLIC_API_URL=https://api.example.com
NEXT_PUBLIC_ENVIRONMENT=production
```

## Branch Strategy

### Production Branch
- **main/master**: Primary production deployment
- Automatic builds on push
- Custom domain assignment

### Development Branch
- **develop**: Staging environment
- Preview deployments
- Testing before production

### Feature Branches
- **feature/***: Preview deployments
- Pull request previews
- Isolated testing environments

## Custom Domains

1. **Add Domain in Amplify Console**
   - Go to your app in the Amplify console
   - Navigate to "Domain management"
   - Add your custom domain

2. **Configure DNS**
   - Update your domain's DNS settings
   - Point to the provided Amplify domain

3. **SSL Certificate**
   - Automatically provisioned by AWS
   - Validated through DNS or email

## GitHub Actions Integration

The module can generate GitHub Actions workflows for CI/CD:

```yaml
name: Deploy to AWS Amplify

on:
  push:
    branches: [ main, master ]
  pull_request:
    branches: [ main, master ]

env:
  AWS_REGION: us-east-1
  AMPLIFY_APP_ID: d1234567890

jobs:
  test:
    runs-on: ubuntu-latest
    steps:
    - uses: actions/checkout@v4
    - name: Set up Node.js
      uses: actions/setup-node@v4
      with:
        node-version: '18'
        cache: 'npm'
    - name: Install dependencies
      run: npm ci
    - name: Run tests
      run: npm test
    - name: Build application
      run: npm run build

  deploy:
    needs: test
    runs-on: ubuntu-latest
    if: github.ref == 'refs/heads/main' || github.ref == 'refs/heads/master'
    steps:
    - uses: actions/checkout@v4
    - name: Configure AWS credentials
      uses: aws-actions/configure-aws-credentials@v4
      with:
        aws-access-key-id: ${{ secrets.AWS_ACCESS_KEY_ID }}
        aws-secret-access-key: ${{ secrets.AWS_SECRET_ACCESS_KEY }}
        aws-region: ${{ env.AWS_REGION }}
    - name: Deploy to Amplify
      run: |
        aws amplify start-job \
          --app-id ${{ env.AMPLIFY_APP_ID }} \
          --branch-name main \
          --job-type RELEASE
```

## Troubleshooting

### Common Issues

#### Build Failures

1. **Missing Dependencies**
   ```bash
   # Check package.json
   npm list --depth=0
   ```

2. **Build Script Issues**
   ```bash
   # Verify build script exists
   npm run build
   ```

3. **Environment Variables**
   - Check Amplify console for environment variables
   - Verify variable names match framework requirements

#### Deployment Issues

1. **AWS Credentials**
   ```bash
   # Verify AWS credentials
   aws sts get-caller-identity
   ```

2. **Repository Access**
   - Ensure GitHub repository is public or accessible
   - Verify GitHub token has necessary permissions

3. **Branch Existence**
   ```bash
   # Check if branch exists
   git branch -a
   ```

### Debug Commands

```bash
# Check app status
devops-ai amplify status

# List all apps
devops-ai amplify list-apps

# List branches
devops-ai amplify list-branches --app-id YOUR_APP_ID

# Check AWS credentials
aws configure list
```

## Best Practices

### 1. Branch Strategy
- Use `main` or `master` for production
- Use `develop` for staging
- Use feature branches for development

### 2. Environment Variables
- Use framework-specific prefixes (REACT_APP_, NEXT_PUBLIC_)
- Keep sensitive data in Amplify console
- Use different values for different environments

### 3. Build Optimization
- Use `npm ci` instead of `npm install`
- Enable caching for faster builds
- Optimize bundle size

### 4. Monitoring
- Set up build notifications
- Monitor build times and failures
- Use Amplify console for insights

## Examples

### Complete Workflow

```bash
# 1. Create app
devops-ai-amplify create-app \
  --name my-react-app \
  --repo https://github.com/username/my-react-app

# 2. Configure for React
devops-ai-amplify configure-app --framework react

# 3. Create development branch
devops-ai-amplify create-branch --branch develop

# 4. Deploy main branch
devops-ai-amplify deploy-branch --branch main --wait

# 5. Check status
devops-ai-amplify status
```

### Multi-Branch Deployment

```bash
# Create feature branch
devops-ai-amplify create-branch --branch feature/new-ui

# Deploy feature branch
devops-ai-amplify deploy-branch --branch feature/new-ui

# Deploy to staging
devops-ai-amplify deploy-branch --branch develop

# Deploy to production
devops-ai-amplify deploy-branch --branch main
```

## Contributing

Contributions are welcome! Please see the main [devops-ai repository](https://github.com/cosmicfusionlabs/devops-ai) for contribution guidelines.

## License

This module is part of the devops-ai project and is licensed under the MIT License. 